@@ -40,7 +40,9 @@ _dialect_lookup = {
     "sparksql": ("dialect_sparksql", "sparksql_dialect"),
     "sqlite": ("dialect_sqlite", "sqlite_dialect"),
     "teradata": ("dialect_teradata", "teradata_dialect"),
+    "trino": ("dialect_trino", "trino_dialect"),
     "tsql": ("dialect_tsql", "tsql_dialect"),
+    "vertica": ("dialect_vertica", "vertica_dialect"),
 }
 
 _legacy_dialects = {
@@ -94,3 +96,13 @@ def dialect_selector(s: str) -> Dialect:
     # Expand any callable references at this point.
     # NOTE: The result of .expand() is a new class.
     return dialect.expand()
+
+
+__all__ = [
+    "Dialect",
+    "DialectTuple",
+    "SQLFluffUserError",
+    "load_raw_dialect",
+    "dialect_readout",
+    "dialect_selector",
+]
