@@ -4,7 +4,7 @@ Matchable objects which return individual segments.
 """
 
 from abc import abstractmethod
-from typing import Collection, Optional, Tuple, Type, Union
+from typing import Any, Callable, Collection, Dict, Optional, Sequence, Tuple, Type
 from uuid import uuid4
 
 import regex
@@ -31,11 +31,16 @@ class BaseParser(Matchable):
         optional: bool = False,
         # The following kwargs are passed on to the segment:
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ) -> None:
         self.raw_class = raw_class
-        self.type: str = type or raw_class.type
+        # Store instance_types rather than just type to allow
+        # for multiple possible types to be supported in derivative
+        # classes.
+        self._instance_types: Tuple[str, ...] = (type or raw_class.type,)
         self.optional = optional
         self._trim_chars = trim_chars
+        self._casefold = casefold
         # Generate a cache key
         self._cache_key = uuid4().hex
 
@@ -50,64 +55,24 @@ class BaseParser(Matchable):
         """Return whether this element is optional."""
         return self.optional
 
-    @abstractmethod
-    def _is_first_match(self, segment: BaseSegment) -> bool:
-        """Does the segment provided match according to the current rules."""
-
-    def _make_match_from_segment(self, segment: BaseSegment) -> RawSegment:
-        """Make a MatchResult from the first segment in the given list.
+    def _match_at(self, idx: int) -> MatchResult:
+        """Construct a MatchResult at a given index.
 
         This is a helper function for reuse by other parsers.
         """
-        return self.raw_class(
-            raw=segment.raw,
-            pos_marker=segment.pos_marker,
-            type=self.type,
-            trim_chars=self._trim_chars,
+        segment_kwargs: Dict[str, Any] = {}
+        if self._instance_types:
+            segment_kwargs["instance_types"] = self._instance_types
+        if self._trim_chars:
+            segment_kwargs["trim_chars"] = self._trim_chars
+        if self._casefold:
+            segment_kwargs["casefold"] = self._casefold
+        return MatchResult(
+            matched_slice=slice(idx, idx + 1),
+            matched_class=self.raw_class,
+            segment_kwargs=segment_kwargs,
         )
 
-    def _match_single(self, segment: BaseSegment) -> Optional[RawSegment]:
-        """Match a single segment.
-
-        Used in the context of matching against the first in a sequence.
-
-        NOTE: We try and allow here for fairly efficient matching against
-        segments which have already been matched. In those cases we still
-        check in the same way, but if matched, we don't try and create a
-        new segment, we just return the existing segment unchanged.
-        """
-        # Does it match? If not, return None.
-        if not self._is_first_match(segment):
-            return None
-        # If it does, we might have already matched it. Is it the right type
-        # already? If so, just return it unchanged.
-        if isinstance(segment, self.raw_class) and segment.type == self.type:
-            return segment
-        # Otherwise create a new match segment
-        return self._make_match_from_segment(segment)
-
-    def match(
-        self,
-        segments: Union[BaseSegment, Tuple[BaseSegment, ...]],
-        parse_context: "ParseContext",
-    ) -> MatchResult:
-        """Compare input segments for a match, return a `MatchResult`.
-
-        Note: For matching here, we only consider the *first* element,
-        because we assume that a keyword can only span one raw segment.
-        """
-        # If we've been passed the singular, make it a tuple
-        if isinstance(segments, BaseSegment):
-            segments = (segments,)
-
-        # We're only going to match against the first element
-        if len(segments) >= 1:
-            seg = self._match_single(segments[0])
-            if seg:
-                return MatchResult((seg,), segments[1:])
-
-        return MatchResult.from_unmatched(segments)
-
 
 class TypedParser(BaseParser):
     """An object which matches and returns raw segments based on types."""
@@ -119,34 +84,84 @@ class TypedParser(BaseParser):
         type: Optional[str] = None,
         optional: bool = False,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ) -> None:
+        """Initialize a new instance of the class.
+
+        Args:
+            template (str): The template type.
+            raw_class (Type[RawSegment]): The raw segment class.
+            type (Optional[str]): The type of the instance.
+            optional (bool): Whether the instance is optional.
+            trim_chars (Optional[Tuple[str, ...]]): The characters to trim.
+            casefold: (Optional[Callable[[str],str]]): The default casing used.
+
+        Returns:
+            None
+        """
         # NB: the template in this case is the _target_ type.
         # The type kwarg is the eventual type.
         self.template = template
+        # Pre-calculate the appropriate frozenset for matching later.
+        self._target_types = frozenset((template,))
         super().__init__(
             raw_class=raw_class,
-            # If no type specified we default to the template
-            type=type or template,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
+        # NOTE: We override the instance types after initialising the base
+        # class. We want to ensure that re-matching is possible by ensuring that
+        # the `type` pre-matching is still present post-match even if it's not
+        # part of the natural type hierarchy for the new `raw_class`.
+        # The new `type` becomes the "primary" type, but the template will still
+        # be part of the resulting `class_types`.
+        # We do this here rather than in the base class to keep the dialect-facing
+        # API the same.
+        self._instance_types: Tuple[str, ...] = ()
+        # Primary type if set.
+        if type is not None:
+            self._instance_types += (type,)
+        # New root types
+        if type != raw_class.type:
+            self._instance_types += (raw_class.type,)
+        # Template type (if it's not in the subclasses of the raw_class).
+        if not raw_class.class_is_type(template):
+            self._instance_types += (template,)
 
     def __repr__(self) -> str:
+        """Return a string representation of the TypedParser object."""
         return f"<TypedParser: {self.template!r}>"
 
     def simple(
-        cls, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
+        self, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
     ) -> SimpleHintType:
-        """Does this matcher support a uppercase hash matching route?
+        """Check if the matcher supports uppercase hash matching route.
 
-        TypedParser segment doesn't support matching against raw strings,
-        but it does support it against types.
+        The TypedParser segment does not support matching against raw strings,
+        but it does support matching against types. Matching is done against both the
+        template and the resulting type, to support re-matching.
+
+        Args:
+            parse_context (ParseContext): The parse context.
+            crumbs (Optional[Tuple[str, ...]], optional): The crumbs.
+            Defaults to None.
+
+        Returns:
+            SimpleHintType: A set of target types.
         """
-        return frozenset(), frozenset((cls.template,))
+        return frozenset(), self._target_types
 
-    def _is_first_match(self, segment: BaseSegment) -> bool:
-        """Return true if the type matches the target type."""
-        return segment.is_type(self.template)
+    def match(
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
+        """Match against this matcher."""
+        if segments[idx].is_type(self.template):
+            return self._match_at(idx)
+        return MatchResult.empty_at(idx)
 
 
 class StringParser(BaseParser):
@@ -159,6 +174,7 @@ class StringParser(BaseParser):
         type: Optional[str] = None,
         optional: bool = False,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         self.template = template.upper()
         # Create list version upfront to avoid recreating it multiple times.
@@ -168,6 +184,7 @@ class StringParser(BaseParser):
             type=type,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
 
     def __repr__(self) -> str:
@@ -183,13 +200,20 @@ class StringParser(BaseParser):
         """
         return self._simple, frozenset()
 
-    def _is_first_match(self, segment: BaseSegment) -> bool:
-        """Does the segment provided match according to the current rules."""
-        # Is the target a match and IS IT CODE.
-        # The latter stops us accidentally matching comments.
-        if self.template == segment.raw_upper and segment.is_code:
-            return True
-        return False
+    def match(
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
+        """Match against this matcher.
+
+        NOTE: We check that the segment is also code to avoid matching
+        unexpected comments.
+        """
+        if segments[idx].raw_upper == self.template and segments[idx].is_code:
+            return self._match_at(idx)
+        return MatchResult.empty_at(idx)
 
 
 class MultiStringParser(BaseParser):
@@ -202,6 +226,7 @@ class MultiStringParser(BaseParser):
         type: Optional[str] = None,
         optional: bool = False,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         self.templates = {template.upper() for template in templates}
         # Create list version upfront to avoid recreating it multiple times.
@@ -211,6 +236,7 @@ class MultiStringParser(BaseParser):
             type=type,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
 
     def __repr__(self) -> str:
@@ -226,13 +252,20 @@ class MultiStringParser(BaseParser):
         """
         return self._simple, frozenset()
 
-    def _is_first_match(self, segment: BaseSegment) -> bool:
-        """Does the segment provided match according to the current rules."""
-        # Is the target a match and IS IT CODE.
-        # The latter stops us accidentally matching comments.
-        if segment.is_code and segment.raw_upper in self.templates:
-            return True
-        return False
+    def match(
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
+        """Match against this matcher.
+
+        NOTE: We check that the segment is also code to avoid matching
+        unexpected comments.
+        """
+        if segments[idx].is_code and segments[idx].raw_upper in self.templates:
+            return self._match_at(idx)
+        return MatchResult.empty_at(idx)
 
 
 class RegexParser(BaseParser):
@@ -246,6 +279,7 @@ class RegexParser(BaseParser):
         optional: bool = False,
         anti_template: Optional[str] = None,
         trim_chars: Optional[Tuple[str, ...]] = None,
+        casefold: Optional[Callable[[str], str]] = None,
     ):
         # Store the optional anti-template
         self.template = template
@@ -258,13 +292,14 @@ class RegexParser(BaseParser):
             type=type,
             optional=optional,
             trim_chars=trim_chars,
+            casefold=casefold,
         )
 
     def __repr__(self) -> str:
         return f"<RegexParser: {self.template!r}>"
 
     def simple(
-        cls, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
+        self, parse_context: ParseContext, crumbs: Optional[Tuple[str, ...]] = None
     ) -> None:
         """Does this matcher support a uppercase hash matching route?
 
@@ -272,26 +307,24 @@ class RegexParser(BaseParser):
         """
         return None
 
-    def _is_first_match(self, segment: BaseSegment) -> bool:
-        """Does the segment provided match according to the current rules.
+    def match(
+        self,
+        segments: Sequence["BaseSegment"],
+        idx: int,
+        parse_context: "ParseContext",
+    ) -> MatchResult:
+        """Match against this matcher.
 
-        RegexParser implements its own matching function where
-        we assume that ._template is a r"" string, and is formatted
-        for use directly as a regex. This only matches on a single segment.
+        NOTE: This method uses .raw_upper and so case sensitivity is
+        not supported.
         """
-        if len(segment.raw) == 0:  # pragma: no cover TODO?
-            # If it's of zero length it's probably a meta segment.
-            # In any case, it won't match here.
-            return False
-        # Try the regex. Case sensitivity is not supported.
-        result = self._template.match(segment.raw_upper)
+        _raw = segments[idx].raw_upper
+        result = self._template.match(_raw)
         if result:
             result_string = result.group(0)
             # Check that we've fully matched
-            if result_string == segment.raw_upper:
+            if result_string == _raw:
                 # Check that the anti_template (if set) hasn't also matched
-                if self.anti_template and self._anti_template.match(segment.raw_upper):
-                    return False
-                else:
-                    return True
-        return False
+                if not self.anti_template or not self._anti_template.match(_raw):
+                    return self._match_at(idx)
+        return MatchResult.empty_at(idx)
