@@ -930,6 +930,10 @@ class BaseSegment(metaclass=SegmentMetaclass):
         """Iterate raw segments, mostly for searching."""
         return [item for s in self.segments for item in s.raw_segments]
 
+    def raw_normalized(self, casefold: bool = True) -> str:
+        """Iterate raw segments, return normalized value."""
+        return "".join(seg.raw_normalized(casefold) for seg in self.get_raw_segments())
+
     def iter_segments(
         self, expanding: Optional[Sequence[str]] = None, pass_through: bool = False
     ) -> Iterator["BaseSegment"]:
@@ -1229,6 +1233,15 @@ class BaseSegment(metaclass=SegmentMetaclass):
         """Stub."""
         raise NotImplementedError()
 
+    @classmethod
+    def from_result_segments(
+        cls,
+        result_segments: Tuple[BaseSegment, ...],
+        segment_kwargs: Dict[str, Any],
+    ) -> "BaseSegment":
+        """Create an instance of this class from a tuple of matched segments."""
+        return cls(segments=result_segments, **segment_kwargs)
+
 
 class UnparsableSegment(BaseSegment):
     """This is a segment which can't be parsed. It indicates a error during parsing."""
