@@ -79,6 +79,16 @@ bigquery_dialect.patch_lexer_matchers(
             r"([rR]?[bB]?|[bB]?[rR]?)?('''((?<!\\)(\\{2})*\\'|'{,2}(?!')|[^'])"
             r"*(?<!\\)(\\{2})*'''|'((?<!\\)(\\{2})*\\'|[^'])*(?<!\\)(\\{2})*')",
             CodeSegment,
+            segment_kwargs={
+                "quoted_value": (
+                    r"(?:[rR]?[bB]?|[bB]?[rR]?)?"
+                    r"(?:'''(?<value>(?:(?<!\\)(?:\\{2})*\\'|'{,2}(?!')|[^'])*"
+                    r"(?<!\\)(?:\\{2})*)'''|'(?<value>(?:(?<!\\)(?:\\{2})*\\'"
+                    r"|[^'])*(?<!\\)(?:\\{2})*)')",
+                    "value",
+                ),
+                "escape_replacements": [(r"\\([\\`\"'?])", "\1")],
+            },
         ),
         RegexLexer(
             "double_quote",
@@ -86,6 +96,16 @@ bigquery_dialect.patch_lexer_matchers(
             r'|[^\"])*(?<!\\)(\\{2})*\"\"\"|"((?<!\\)(\\{2})*\\"|[^"])*(?<!\\)'
             r'(\\{2})*")',
             CodeSegment,
+            segment_kwargs={
+                "quoted_value": (
+                    r"([rR]?[bB]?|[bB]?[rR]?)?"
+                    r"(\"\"\"(?<value>((?<!\\)(\\{2})*\\\"|\"{,2}(?!\")"
+                    r'|[^\"])*(?<!\\)(\\{2})*)\"\"\"|"(?<value>((?<!\\)'
+                    r'(\\{2})*\\"|[^"])*(?<!\\)(\\{2})*)")',
+                    "value",
+                ),
+                "escape_replacements": [(r"\\([\\`\"'?])", "\1")],
+            },
         ),
     ]
 )
@@ -149,6 +169,33 @@ bigquery_dialect.add(
         r"[A-Z0-9_]+",
         IdentifierSegment,
         type="naked_identifier",
+        casefold=str.upper,
+    ),
+    NakedCSIdentifierPart=RegexParser(
+        # Same as NakedIdentifierPart, but case-sensitive.
+        r"[A-Z0-9_]+",
+        IdentifierSegment,
+        type="naked_identifier",
+    ),
+    NakedCSIdentifierSegment=SegmentGenerator(
+        # Generate the anti template from the set of reserved keywords
+        lambda dialect: RegexParser(
+            r"[A-Z_][A-Z0-9_]*",
+            IdentifierSegment,
+            type="naked_identifier",
+            anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+        )
+    ),
+    QuotedCSIdentifierSegment=TypedParser(
+        "back_quote",
+        IdentifierSegment,
+        type="quoted_identifier",
+        trim_chars=("`",),
+    ),
+    SingleCSIdentifierGrammar=OneOf(
+        Ref("NakedCSIdentifierSegment"),
+        Ref("QuotedCSIdentifierSegment"),
+        terminators=[Ref("DotSegment")],
     ),
     SingleIdentifierFullGrammar=OneOf(
         Ref("NakedIdentifierSegment"),
@@ -213,6 +260,7 @@ bigquery_dialect.replace(
             IdentifierSegment,
             type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
+            casefold=str.upper,
         )
     ),
     FunctionContentsExpressionGrammar=OneOf(
@@ -339,6 +387,22 @@ class ArrayTypeSegment(ansi.ArrayTypeSegment):
     )
 
 
+class ForSystemTimeAsOfSegment(BaseSegment):
+    """A `FOR SYSTEM_TIME AS OF` syntax.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/query-syntax#for_system_time_as_of
+    """
+
+    type = "for_system_time_as_of_segment"
+    match_grammar = Sequence(
+        "FOR",
+        OneOf("SYSTEM_TIME", Sequence("SYSTEM", "TIME")),
+        "AS",
+        "OF",
+        Ref("ExpressionSegment"),
+    )
+
+
 class QualifyClauseSegment(BaseSegment):
     """A `QUALIFY` clause like in `SELECT`."""
 
@@ -435,6 +499,8 @@ class StatementSegment(ansi.StatementSegment):
             Ref("SetStatementSegment"),
             Ref("ExportStatementSegment"),
             Ref("CreateExternalTableStatementSegment"),
+            Ref("CreateSnapshotTableStatementSegment"),
+            Ref("ExecuteImmediateSegment"),
             Ref("AssertStatementSegment"),
             Ref("CallStatementSegment"),
             Ref("ReturnStatementSegment"),
@@ -443,9 +509,19 @@ class StatementSegment(ansi.StatementSegment):
             Ref("ContinueStatementSegment"),
             Ref("RaiseStatementSegment"),
             Ref("AlterViewStatementSegment"),
+            Ref("AlterSchemaStatementSegment"),
             Ref("CreateMaterializedViewStatementSegment"),
+            Ref("CreateMaterializedViewAsReplicaOfStatementSegment"),
             Ref("AlterMaterializedViewStatementSegment"),
             Ref("DropMaterializedViewStatementSegment"),
+            Ref("DropProcedureStatementSegment"),
+            Ref("UndropSchemaStatementSegment"),
+            Ref("CreateSearchIndexStatementSegment"),
+            Ref("CreateVectorIndexStatementSegment"),
+            Ref("CreateRowAccessPolicyStatementSegment"),
+            Ref("CreateCapacityStatementSegment"),
+            Ref("CreateReservationStatementSegment"),
+            Ref("CreateAssignmentStatementSegment"),
         ],
     )
 
@@ -484,7 +560,7 @@ class ForInStatementsSegment(BaseSegment):
             Ref("DelimiterGrammar"),
         ),
         terminators=[Sequence("END", "FOR")],
-        parse_mode=ParseMode.GREEDY,
+        reset_terminators=True,
     )
 
 
@@ -527,7 +603,7 @@ class RepeatStatementsSegment(BaseSegment):
             Ref("DelimiterGrammar"),
         ),
         terminators=["UNTIL"],
-        parse_mode=ParseMode.GREEDY,
+        reset_terminators=True,
     )
 
 
@@ -570,7 +646,7 @@ class IfStatementsSegment(BaseSegment):
             "ELSEIF",
             Sequence("END", "IF"),
         ],
-        parse_mode=ParseMode.GREEDY,
+        reset_terminators=True,
     )
 
 
@@ -626,7 +702,7 @@ class LoopStatementsSegment(BaseSegment):
             Ref("DelimiterGrammar"),
         ),
         terminators=[Sequence("END", "LOOP")],
-        parse_mode=ParseMode.GREEDY,
+        reset_terminators=True,
     )
 
 
@@ -660,7 +736,7 @@ class WhileStatementsSegment(BaseSegment):
             Ref("DelimiterGrammar"),
         ),
         terminators=[Sequence("END", "WHILE")],
-        parse_mode=ParseMode.GREEDY,
+        reset_terminators=True,
     )
 
 
@@ -718,6 +794,7 @@ bigquery_dialect.replace(
         IdentifierSegment,
         type="quoted_identifier",
         trim_chars=("`",),
+        casefold=str.upper,
     ),
     # Add ParameterizedSegment to the ansi NumericLiteralSegment
     NumericLiteralSegment=OneOf(
@@ -736,14 +813,7 @@ bigquery_dialect.replace(
         ]
     ),
     PostTableExpressionGrammar=Sequence(
-        Sequence(
-            "FOR",
-            OneOf("SYSTEM_TIME", Sequence("SYSTEM", "TIME")),
-            "AS",
-            "OF",
-            Ref("ExpressionSegment"),
-            optional=True,
-        ),
+        Ref("ForSystemTimeAsOfSegment", optional=True),
         Sequence(
             "WITH",
             "OFFSET",
@@ -1004,6 +1074,7 @@ class FunctionDefinitionGrammar(ansi.FunctionDefinitionGrammar):
                     ),
                 ),
             ),
+            Ref("OptionsSegment", optional=True),
         )
     )
 
@@ -1031,6 +1102,16 @@ class ExceptClauseSegment(BaseSegment):
     )
 
 
+class TransactionStatementSegment(ansi.TransactionStatementSegment):
+    """A `BEGIN`, `COMMIT`, or `ROLLBACK` statement."""
+
+    match_grammar = Sequence(
+        OneOf("BEGIN", "COMMIT", "ROLLBACK"),
+        Ref.keyword("TRANSACTION", optional=True),
+        terminators=[Ref("DelimiterGrammar")],
+    )
+
+
 class BeginStatementSegment(BaseSegment):
     """A `BEGIN...EXCEPTION...END` statement.
 
@@ -1040,37 +1121,45 @@ class BeginStatementSegment(BaseSegment):
     type = "begin_statement"
 
     match_grammar = Sequence(
-        "BEGIN",
-        Indent,
-        AnyNumberOf(
-            Sequence(
-                Ref("StatementSegment"),
-                Ref("DelimiterGrammar"),
-            ),
-            min_times=1,
-            terminators=["END", "EXCEPTION"],
-            parse_mode=ParseMode.GREEDY,
-        ),
-        Dedent,
         Sequence(
-            "EXCEPTION",
-            "WHEN",
-            "ERROR",
-            "THEN",
+            Ref("SingleIdentifierFullGrammar"), Ref("ColonSegment"), optional=True
+        ),
+        "BEGIN",
+        Sequence(
             Indent,
             AnyNumberOf(
                 Sequence(
-                    Ref("StatementSegment"),
+                    OneOf(Ref("StatementSegment"), Ref("MultiStatementSegment")),
                     Ref("DelimiterGrammar"),
                 ),
+                # We can't terminate on `END` due to possible nesting
+                terminators=["EXCEPTION"],
+                reset_terminators=True,
                 min_times=1,
-                terminators=["END"],
-                parse_mode=ParseMode.GREEDY,
             ),
             Dedent,
+            Sequence(
+                "EXCEPTION",
+                "WHEN",
+                "ERROR",
+                "THEN",
+                Indent,
+                AnyNumberOf(
+                    Sequence(
+                        OneOf(Ref("StatementSegment"), Ref("MultiStatementSegment")),
+                        Ref("DelimiterGrammar"),
+                    ),
+                    min_times=1,
+                    # We can't terminate on `END` due to possible nesting
+                    reset_terminators=True,
+                ),
+                Dedent,
+                optional=True,
+            ),
             optional=True,
         ),
         "END",
+        Ref("SingleIdentifierFullGrammar", optional=True),
     )
 
 
@@ -1298,11 +1387,11 @@ class TableReferenceSegment(ansi.ObjectReferenceSegment):
 
     match_grammar: Matchable = Delimited(
         Sequence(
-            Ref("SingleIdentifierGrammar"),
+            Ref("SingleCSIdentifierGrammar"),
             AnyNumberOf(
                 Sequence(
                     Ref("DashSegment"),
-                    Ref("NakedIdentifierPart"),
+                    Ref("NakedCSIdentifierPart"),
                     allow_gaps=False,
                 ),
                 optional=True,
@@ -1444,6 +1533,40 @@ class SetStatementSegment(BaseSegment):
     )
 
 
+class ExecuteImmediateSegment(BaseSegment):
+    """An EXECUTE IMMEDIATE statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/procedural-language#execute_immediate
+    """
+
+    type = "execute_immediate"
+    match_grammar = Sequence(
+        "EXECUTE",
+        "IMMEDIATE",
+        OptionallyBracketed(
+            OneOf(
+                Ref("QuotedLiteralSegment"),  # String
+                Ref("SingleIdentifierFullGrammar"),  # Variable
+                Ref("FunctionSegment"),  # Function
+                Ref("CaseExpressionSegment"),  # Conditional Expression
+                Bracketed(Ref("SelectableGrammar")),  # Expression Subquery
+            )
+        ),
+        Sequence("INTO", Delimited(Ref("SingleIdentifierFullGrammar")), optional=True),
+        Sequence(
+            "USING",
+            Delimited(
+                Sequence(
+                    Ref("BaseExpressionElementGrammar"),
+                    # The `AS` is required when using an alias in this context
+                    Sequence("AS", Ref("SingleIdentifierFullGrammar"), optional=True),
+                ),
+            ),
+            optional=True,
+        ),
+    )
+
+
 class PartitionBySegment(BaseSegment):
     """PARTITION BY partition_expression."""
 
@@ -1466,6 +1589,38 @@ class ClusterBySegment(BaseSegment):
     )
 
 
+class DefaultCollateSegment(BaseSegment):
+    """DEFAULT COLLATE clause for a table or a dataset.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/collation-concepts#default_collation
+    """
+
+    type = "default_collate"
+    match_grammar: Matchable = Sequence(
+        "DEFAULT",
+        "COLLATE",
+        Ref("LiteralGrammar"),
+    )
+
+
+class GrantToSegment(BaseSegment):
+    """GRANT TO (grantee_list).
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_row_access_policy_statement
+    """
+
+    type = "grant_to_segment"
+    match_grammar: Matchable = Sequence(
+        "GRANT",
+        "TO",
+        Bracketed(
+            Delimited(
+                Ref("QuotedLiteralSegment"),
+            ),
+        ),
+    )
+
+
 class OptionsSegment(BaseSegment):
     """OPTIONS clause for a table."""
 
@@ -1534,6 +1689,22 @@ class ViewColumnDefinitionSegment(ansi.ColumnDefinitionSegment):
     )
 
 
+class CreateSchemaStatementSegment(ansi.CreateSchemaStatementSegment):
+    """A `CREATE SCHEMA` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_schema_statement
+    """
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        "SCHEMA",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        Ref("DefaultCollateSegment", optional=True),
+        Ref("OptionsSegment", optional=True),
+    )
+
+
 class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
     """`CREATE TABLE` statement.
 
@@ -1550,6 +1721,7 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
         Sequence(
             OneOf("COPY", "LIKE", "CLONE"),
             Ref("TableReferenceSegment"),
+            Ref("ForSystemTimeAsOfSegment", optional=True),
             optional=True,
         ),
         # Column list
@@ -1565,6 +1737,7 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
             ),
             optional=True,
         ),
+        Ref("DefaultCollateSegment", optional=True),
         Ref("PartitionBySegment", optional=True),
         Ref("ClusterBySegment", optional=True),
         Ref("OptionsSegment", optional=True),
@@ -1601,6 +1774,50 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
                 ),
                 allow_trailing=True,
             ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_add_foreign_key_statement
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_add_primary_key_statement
+            Delimited(
+                OneOf(
+                    Sequence(
+                        "ADD",
+                        Sequence(
+                            "CONSTRAINT",
+                            Ref("IfNotExistsGrammar", optional=True),
+                            Ref("SingleIdentifierGrammar"),
+                            optional=True,
+                        ),
+                        "FOREIGN",
+                        "KEY",
+                        Bracketed(
+                            Delimited(
+                                Ref("SingleIdentifierGrammar"),
+                            ),
+                        ),
+                        "REFERENCES",
+                        Ref("TableReferenceSegment"),
+                        Bracketed(
+                            Delimited(
+                                Ref("SingleIdentifierGrammar"),
+                            ),
+                        ),
+                        "NOT",
+                        "ENFORCED",
+                    ),
+                    Sequence(
+                        "ADD",
+                        "PRIMARY",
+                        "KEY",
+                        Bracketed(
+                            Delimited(
+                                Ref("SingleIdentifierGrammar"),
+                            ),
+                        ),
+                        "NOT",
+                        "ENFORCED",
+                    ),
+                ),
+                allow_trailing=True,
+            ),
             # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_table_rename_to_statement
             Sequence(
                 "RENAME",
@@ -1662,6 +1879,43 @@ class AlterTableStatementSegment(ansi.AlterTableStatementSegment):
     )
 
 
+class AlterSchemaStatementSegment(BaseSegment):
+    """A `ALTER SCHEMA` statement."""
+
+    type = "alter_schema_statement"
+
+    match_grammar: Matchable = Sequence(
+        "ALTER",
+        "SCHEMA",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        OneOf(
+            Sequence(
+                "SET",
+                OneOf(
+                    # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_schema_collate_statement
+                    Ref("DefaultCollateSegment"),
+                    # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_schema_set_options_statement
+                    Ref("OptionsSegment"),
+                ),
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_schema_add_replica_statement
+            Sequence(
+                "ADD",
+                "REPLICA",
+                Ref("BaseExpressionElementGrammar"),
+                Ref("OptionsSegment", optional=True),
+            ),
+            # https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#alter_schema_drop_replica_statement
+            Sequence(
+                "DROP",
+                "REPLICA",
+                Ref("BaseExpressionElementGrammar"),
+            ),
+        ),
+    )
+
+
 class CreateExternalTableStatementSegment(BaseSegment):
     """A `CREATE EXTERNAL TABLE` statement.
 
@@ -1707,6 +1961,27 @@ class CreateExternalTableStatementSegment(BaseSegment):
     )
 
 
+class CreateSnapshotTableStatementSegment(BaseSegment):
+    """A `CREATE SNAPSHOT TABLE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_snapshot_table_statement
+    """
+
+    type = "create_snapshot_table_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        "SNAPSHOT",
+        "TABLE",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("TableReferenceSegment"),
+        "CLONE",
+        Ref("TableReferenceSegment"),
+        Ref("ForSystemTimeAsOfSegment", optional=True),
+        Ref("OptionsSegment", optional=True),
+    )
+
+
 class CreateViewStatementSegment(ansi.CreateViewStatementSegment):
     """A `CREATE VIEW` statement.
 
@@ -1773,6 +2048,27 @@ class CreateMaterializedViewStatementSegment(BaseSegment):
     )
 
 
+class CreateMaterializedViewAsReplicaOfStatementSegment(BaseSegment):
+    """A `CREATE MATERIALIZED VIEW AS REPLICA OF` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_materialized_view_as_replica_of_statement
+    """
+
+    type = "create_materialized_view_as_replica_of_statement"
+
+    match_grammar = Sequence(
+        "CREATE",
+        "MATERIALIZED",
+        "VIEW",
+        Ref("TableReferenceSegment"),
+        Ref("OptionsSegment", optional=True),
+        "AS",
+        "REPLICA",
+        "OF",
+        Ref("TableReferenceSegment"),
+    )
+
+
 class AlterMaterializedViewStatementSegment(BaseSegment):
     """A `ALTER MATERIALIZED VIEW SET OPTIONS` statement.
 
@@ -1792,6 +2088,74 @@ class AlterMaterializedViewStatementSegment(BaseSegment):
     )
 
 
+class DropTableStatementSegment(BaseSegment):
+    """A `DROP [SNAPSHOT | EXTERNAL] TABLE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_table_statement
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_snapshot_table_statement
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_external_table_statement
+    """
+
+    type = "drop_table_statement"
+
+    match_grammar: Matchable = Sequence(
+        "DROP",
+        OneOf("SNAPSHOT", "EXTERNAL", optional=True),
+        "TABLE",
+        Ref("IfExistsGrammar", optional=True),
+        Delimited(Ref("TableReferenceSegment")),
+    )
+
+
+class DropFunctionStatementSegment(BaseSegment):
+    """A `DROP [TABLE] FUNCTION` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_function_statement
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_table_function
+    """
+
+    type = "drop_function_statement"
+
+    match_grammar = Sequence(
+        "DROP",
+        Sequence("TABLE", optional=True),
+        "FUNCTION",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("FunctionNameSegment"),
+    )
+
+
+class DropProcedureStatementSegment(BaseSegment):
+    """A `DROP PROCEDURE` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#drop_procedure_statement
+    """
+
+    type = "drop_procedure_statement"
+
+    match_grammar = Sequence(
+        "DROP",
+        "PROCEDURE",
+        Ref("IfExistsGrammar", optional=True),
+        Ref("ProcedureNameSegment"),
+    )
+
+
+class UndropSchemaStatementSegment(BaseSegment):
+    """A `UNDROP SCHEMA` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#undrop_schema_statement
+    """
+
+    type = "undrop_schema_statement"
+    match_grammar: Matchable = Sequence(
+        "UNDROP",
+        "SCHEMA",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("SchemaReferenceSegment"),
+    )
+
+
 class DropMaterializedViewStatementSegment(BaseSegment):
     """A `DROP MATERIALIZED VIEW` statement.
 
@@ -2096,14 +2460,29 @@ class ExportStatementSegment(BaseSegment):
                             CodeSegment,
                             type="export_option",
                         ),
+                        StringParser(
+                            "header",
+                            CodeSegment,
+                            type="export_option",
+                        ),
+                        StringParser(
+                            "overwrite",
+                            CodeSegment,
+                            type="export_option",
+                        ),
                         StringParser(
                             "uri",
                             CodeSegment,
                             type="export_option",
                         ),
+                        StringParser(
+                            "use_avro_logical_types",
+                            CodeSegment,
+                            type="export_option",
+                        ),
                     ),
                     Ref("EqualsSegment"),
-                    Ref("QuotedLiteralSegment"),
+                    Ref("BaseExpressionElementGrammar"),
                 ),
                 # Bool options
                 # Note: adding as own type, rather than keywords as convention with
@@ -2171,23 +2550,6 @@ class ProcedureParameterListSegment(BaseSegment):
     )
 
 
-class ProcedureStatements(BaseSegment):
-    """Statements within a CREATE PROCEDURE statement.
-
-    https://cloud.google.com/bigquery/docs/procedures
-    """
-
-    type = "procedure_statements"
-    match_grammar = AnyNumberOf(
-        Sequence(
-            Ref("StatementSegment"),
-            Ref("DelimiterGrammar"),
-        ),
-        terminators=["END"],
-        parse_mode=ParseMode.GREEDY,
-    )
-
-
 class CreateProcedureStatementSegment(BaseSegment):
     """A `CREATE PROCEDURE` statement.
 
@@ -2203,19 +2565,8 @@ class CreateProcedureStatementSegment(BaseSegment):
         Ref("IfNotExistsGrammar", optional=True),
         Ref("ProcedureNameSegment"),
         Ref("ProcedureParameterListSegment"),
-        Sequence(
-            "OPTIONS",
-            "strict_mode",
-            StringParser("strict_mode", CodeSegment, type="procedure_option"),
-            Ref("EqualsSegment"),
-            Ref("BooleanLiteralGrammar"),
-            optional=True,
-        ),
-        "BEGIN",
-        Indent,
-        Ref("ProcedureStatements"),
-        Dedent,
-        "END",
+        Ref("OptionsSegment", optional=True),
+        Ref("BeginStatementSegment", reset_terminators=True),
     )
 
 
@@ -2310,3 +2661,130 @@ class RaiseStatementSegment(BaseSegment):
             optional=True,
         ),
     )
+
+
+class CreateSearchIndexStatementSegment(BaseSegment):
+    """A `CREATE SEARCH INDEX` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_search_index_statement
+    """
+
+    type = "create_search_index_statement"
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        "SEARCH",
+        "INDEX",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("IndexReferenceSegment"),
+        "ON",
+        Ref("TableReferenceSegment"),
+        Bracketed(
+            OneOf(
+                Sequence("ALL", "COLUMNS"),
+                Delimited(
+                    Ref("IndexColumnDefinitionSegment"),
+                ),
+            )
+        ),
+        Ref("OptionsSegment", optional=True),
+    )
+
+
+class CreateVectorIndexStatementSegment(BaseSegment):
+    """A `CREATE VECTOR INDEX` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_vector_index_statement
+    """
+
+    type = "create_vector_index_statement"
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "VECTOR",
+        "INDEX",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("IndexReferenceSegment"),
+        "ON",
+        Ref("TableReferenceSegment"),
+        Bracketed(
+            Delimited(
+                Ref("IndexColumnDefinitionSegment"),
+            ),
+        ),
+        Ref("OptionsSegment"),
+    )
+
+
+class CreateRowAccessPolicyStatementSegment(BaseSegment):
+    """A `CREATE ROW ACCESS POLICY` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_row_access_policy_statement
+    """
+
+    type = "create_row_access_policy_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        Ref("OrReplaceGrammar", optional=True),
+        "ROW",
+        "ACCESS",
+        "POLICY",
+        Ref("IfNotExistsGrammar", optional=True),
+        Ref("NakedIdentifierSegment"),  # row_access_policy_name
+        "ON",
+        Ref("TableReferenceSegment"),
+        Ref("GrantToSegment", optional=True),
+        "FILTER",
+        "USING",
+        Bracketed(
+            Ref("ExpressionSegment"),
+        ),
+    )
+
+
+class CreateCapacityStatementSegment(BaseSegment):
+    """A `CREATE CAPACITY` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_capacity_statement
+    """
+
+    type = "create_capacity_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        "CAPACITY",
+        Ref("TableReferenceSegment"),
+        Ref("OptionsSegment"),
+    )
+
+
+class CreateReservationStatementSegment(BaseSegment):
+    """A `CREATE RESERVATION` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_reservation_statement
+    """
+
+    type = "create_reservation_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        "RESERVATION",
+        Ref("TableReferenceSegment"),
+        Ref("OptionsSegment"),
+    )
+
+
+class CreateAssignmentStatementSegment(BaseSegment):
+    """A `CREATE ASSIGNMENT` statement.
+
+    https://cloud.google.com/bigquery/docs/reference/standard-sql/data-definition-language#create_assignment_statement
+    """
+
+    type = "create_assignment_statement"
+
+    match_grammar: Matchable = Sequence(
+        "CREATE",
+        "ASSIGNMENT",
+        Ref("TableReferenceSegment"),
+        Ref("OptionsSegment"),
+    )
