@@ -13,6 +13,7 @@ BETWEEN
 BY
 CASE
 CAST
+CLONE
 COLLATE
 CONTAINS
 CREATE
@@ -101,12 +102,14 @@ WITHIN"""
 
 # Note BigQuery doesn't have a list of Unreserved Keywords
 # so these are just ones we need to allow parsing to work
-bigquery_unreserved_keywords = """ACCOUNT
+bigquery_unreserved_keywords = """ACCESS
+ACCOUNT
 ADD
 ADMIN
 AFTER
 ALTER
 APPLY
+ASSIGNMENT
 ASSERT
 AUTO_INCREMENT
 BEGIN
@@ -116,6 +119,7 @@ BINDING
 BREAK
 CACHE
 CALL
+CAPACITY
 CASCADE
 CHAIN
 CHARACTER
@@ -169,6 +173,7 @@ GRANTED
 GRANTS
 HOUR
 ILIKE
+IMMEDIATE
 IMPORTED
 IN
 INCREMENT
@@ -236,6 +241,8 @@ RENAME
 REPEAT
 REPEATABLE
 REPLACE
+REPLICA
+RESERVATION
 RESOURCE
 RESTRICT
 RETURN
@@ -250,12 +257,14 @@ SAFE
 SATURDAY
 SCHEMA
 SCHEMAS
+SEARCH
 SECOND
 SEPARATOR
 SERVER
 SEQUENCE
 SESSION_USER
 SHARE
+SNAPSHOT
 SOURCE
 STAGE
 START
@@ -278,6 +287,7 @@ TRIGGER
 TRUNCATE
 TUESDAY
 TYPE
+UNDROP
 UNIQUE
 UNSIGNED
 UNTIL
@@ -289,6 +299,7 @@ USER
 VALUE
 VALUES
 VARYING
+VECTOR
 VERSION
 VIEW
 WAREHOUSE
