@@ -12,6 +12,7 @@ from sqlfluff.core.parser import (
     CodeSegment,
     Dedent,
     Delimited,
+    IdentifierSegment,
     Indent,
     Matchable,
     Nothing,
@@ -19,10 +20,12 @@ from sqlfluff.core.parser import (
     OptionallyBracketed,
     ParseMode,
     Ref,
+    RegexLexer,
     Sequence,
     StringLexer,
     StringParser,
     SymbolSegment,
+    TypedParser,
 )
 from sqlfluff.dialects import dialect_ansi as ansi
 from sqlfluff.dialects import dialect_postgres as postgres
@@ -55,11 +58,6 @@ duckdb_dialect.add(
 )
 
 duckdb_dialect.replace(
-    SingleIdentifierGrammar=OneOf(
-        Ref("NakedIdentifierSegment"),
-        Ref("QuotedIdentifierSegment"),
-        Ref("SingleQuotedIdentifierSegment"),
-    ),
     DivideSegment=OneOf(
         StringParser("//", BinaryOperatorSegment),
         StringParser("/", BinaryOperatorSegment),
@@ -114,6 +112,13 @@ duckdb_dialect.replace(
             parse_mode=ParseMode.GREEDY,
         ),
     ),
+    # Matching postgres lower casefold, as it is case-insensitive
+    QuotedIdentifierSegment=TypedParser(
+        "double_quote", IdentifierSegment, type="quoted_identifier", casefold=str.lower
+    ),
+    SingleQuotedIdentifierSegment=TypedParser(
+        "single_quote", IdentifierSegment, type="quoted_identifier", casefold=str.lower
+    ),
 )
 
 duckdb_dialect.insert_lexer_matchers(
@@ -123,6 +128,31 @@ duckdb_dialect.insert_lexer_matchers(
     before="divide",
 )
 
+duckdb_dialect.patch_lexer_matchers(
+    [
+        # In DuckDB, a double single/double quote resolves as a single/double quote in
+        # the string.
+        RegexLexer(
+            "single_quote",
+            r"'([^']|'')*'",
+            CodeSegment,
+            segment_kwargs={
+                "quoted_value": (r"'((?:[^']|'')*)'", 1),
+                "escape_replacements": [(r"''", "'")],
+            },
+        ),
+        RegexLexer(
+            "double_quote",
+            r'"([^"]|"")*"',
+            CodeSegment,
+            segment_kwargs={
+                "quoted_value": (r'"((?:[^"]|"")*)"', 1),
+                "escape_replacements": [(r'""', '"')],
+            },
+        ),
+    ]
+)
+
 
 class ColumnConstraintSegment(ansi.ColumnConstraintSegment):
     """A column option; each CREATE TABLE column can have 0 or more.
@@ -363,6 +393,26 @@ class UnorderedSelectStatementSegment(ansi.UnorderedSelectStatementSegment):
     )
 
 
+class TableReferenceSegment(ansi.TableReferenceSegment):
+    """A reference to an table, CTE, subquery or alias.
+
+    Overload for DuckDB as only tables can be single quoted identifiers
+    when used by the httpfs extension.
+    """
+
+    match_grammar = ansi.TableReferenceSegment.match_grammar.copy(
+        insert=[Ref("SingleQuotedIdentifierSegment")],
+    )
+
+
+class AliasExpressionSegment(ansi.AliasExpressionSegment):
+    """A reference to an object with an `AS` clause.
+
+    The optional AS keyword allows both implicit and explicit aliasing.
+    Inherit from ANSI instead of Postgres.
+    """
+
+
 class OrderByClauseSegment(ansi.OrderByClauseSegment):
     """A `ORDER BY` clause like in `SELECT`."""
 
