@@ -258,6 +258,8 @@ UNRESERVED_KEYWORDS = [
     "TEMPORARY",
     "TERMINATED",
     "TIME",
+    "TIMESTAMP_LTZ",
+    "TIMESTAMP_NTZ",
     "TOUCH",
     "TRANSACTION",
     "TRANSACTIONS",
